import sys
from typing import Callable, Dict, Any, List
import requests
from .helpers import check_required_env_vars, combine_env_configs


env_config: Dict[str, Any] = combine_env_configs()


def make_request(
    method: str,
    url: str,
    headers: Dict[str, str] = None,
    params: Dict[str, Any] = None,
    data: Dict[str, Any] = None,
    json: Dict[str, Any] = None
) -> requests.Response:
    """Perform an HTTP request on behalf of a calling function

    Args:
        method (str): the HTTP method to use
        url (str): the API URL to call
        headers (Dict[str, str], optional): the HTTP headers to use in the request. Defaults to None.
        params (Dict[str, Any], optional): the query parameters to use in the request. Defaults to None.
        data (Dict[str, Any], optional): the data to use in the request. Defaults to None.
        json (Dict[str, Any], optional): the json data to use in the request. Defaults to None.

    Raises:
        ValueError: this will raise if an invalid HTTP method is passed

    Returns:
        requests.Response: the HTTP response from the request
    """

    # Define required environment variables
    required_vars: List[str] = []

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    proxies: Dict[str, Any] = {
        'http': env_config['HTTP_PROXY'] if 'HTTP_PROXY' in env_config else '',
        'https': env_config['HTTPS_PROXY'] if 'HTTPS_PROXY' in env_config else ''
    }

    verify: bool = False if 'VERIFY_SSL' in env_config and env_config['VERIFY_SSL'].lower() == 'false' else True
    if verify is False:
        import urllib3
        urllib3.disable_warnings()

    method_map: Dict[str, Callable] = {
        'GET': requests.get,
        'POST': requests.post,
        'PUT': requests.put,
        'DELETE': requests.delete,
        'PATCH': requests.patch
    }

    request_func = method_map.get(method.upper())
    if not request_func:
        raise ValueError(f'Unsupported HTTP method: {method}')

    return request_func(url, headers=headers, params=params, data=data, json=json, proxies=proxies, verify=verify)



import sys
from typing import Dict, Any, List
from requests import Response
from .broker import make_request
from .helpers import check_required_env_vars, combine_env_configs


env_config: Dict[str, Any] = combine_env_configs()


def spycloud_sip_cookie_domains(cookie_domains: str, **kwargs: Dict[str, Any]) -> Response:
    """Return botnet sourced cookie data for your domain and its subdomains

    Args:
        cookie_domains (str): This parameter allows you to define a cookie \
            domain to search against, results will include all subdomains. \
            Optionally, a specific cookie subdomain could be used which will \
            result in only that specific cookie subdomain returned.

    Returns:
        Response: requests.Response json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_SIP_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sip-v1/breach/data/cookie-domains/{cookie_domains}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_SIP_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result

def spycloud_ato_email(email_addresses:str, **kwargs: Dict[str, Any]) -> Response:
    """Return account takeover (ATO) breach data related to a comma-separated string of emails

    Args:
        email_addresses (str): a comma-separated list of email addresses (limit of 10 at a time)

    Returns:
        Response: requests.Respone json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sp-v2/breach/data/emails/{email_addresses}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result

def spycloud_ato_ip(ip_address:str, **kwargs: Dict[str, Any]) -> Response:
    """Return account takeover (ATO) breach data related to an IP address

    Args:
        ip_address (str): IP address or network CIDR notation to search \
            for. For CIDR notation, use an underscore instead of a slash.

    Returns:
        Response: requests.Respone json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sp-v2/breach/data/ips/{ip_address}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result

def spycloud_ato_username(username:str, **kwargs: Dict[str, Any]) -> Response:
    """Return account takeover (ATO) breach data related to a username

    Args:
        username (str): Username you wish to search for. You can also \
            search for the sha1, sha256, or sha512 hash of the username.

    Returns:
        Response: requests.Respone json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sp-v2/breach/data/usernames/{username}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result

def spycloud_ato_phone_number(phone_number:str, **kwargs: Dict[str, Any]) -> Response:
    """Return account takeover (ATO) breach data related to a phone number

    Args:
        username (str): phone number you wish to search for. Must only be \
            numerical values of length 7 to 15 characters. You can also \
            search for the sha1, sha256, or sha512 hash of the phone number.

    Returns:
        Response: requests.Respone json response from the request
    """

    # Define required environment variables
    required_vars: List[str] = [
        'SPYCLOUD_API_ATO_KEY'
    ]

    # Check and ensure that required variables are present, exits if not
    check_required_env_vars(env_config, required_vars)

    method: str = 'get'
    url: str = f'https://api.spycloud.io/sp-v2/breach/data/phone-numbers/{phone_number}'
    headers: Dict = {
        'accept': 'application/json',
        'x-api-key': env_config['SPYCLOUD_API_ATO_KEY']
    }
    params: Dict = dict(kwargs)

    result: Response = make_request(method=method, url=url, headers=headers, params=params)

    return result




from dotenv import dotenv_values, find_dotenv
import os
import sys
from typing import Dict, Set, List, Any


def check_required_env_vars(config: Dict[str, str], required_vars: List[str]) -> None:
    """Ensure that the env variables required for a function are present either in \
        the .env file, or in the system's environment variables.

    Args:
        config (Dict[str, str]): the env_config variable that contains values from the .env file
        required_vars (List[str]): the env variables required for a function to successfully function
    """

    dotenv_missing_vars: Set[str] = set(required_vars) - set(config.keys())
    osenv_missing_vars: Set[str] = set(required_vars) - set(os.environ)
    missing_vars = dotenv_missing_vars | osenv_missing_vars

    if dotenv_missing_vars and osenv_missing_vars:
        print(f'[!] Error: missing required environment variables: {", ".join(missing_vars)}. '
              'Please ensure these are present either in your .env file, or in the '
              'system\'s environment variables.', file=sys.stderr)
        sys.exit(1)

def combine_env_configs() -> Dict[str, Any]:
    """_summary_

    Args:
        dotenv_config (Dict[str, str]): _description_
        osenv_config (Dict[str, str]): _description_

    Returns:
        Dict: _description_
    """

    env_config: Dict[str, Any] = dict(dotenv_values(find_dotenv()))

    combined_config: Dict[str, Any] = {**env_config, **dict(os.environ)}

    return combined_config